<?php

namespace Countee\Model\Bank;

use Countee;
use GuzzleHttp;
use Nette;

abstract class Account extends Countee\Model\BaseDatabaseModel implements Countee\Model\Bank\IAccount
{
    /**
     * Sync limit in seconds
     */
    const SYNC_LIMIT = 30;
    
    /**
     * @var Countee\Model\Payable\Receivable
     */
    protected $receivable;
    
    /**
     * @var Countee\Model\Payable\Commitment
     */
    protected $commitment;
    
    /**
     * Constructor
     * @param Nette\Database\Context $database
     * @param Countee\Model\Payable\Receivable $receivable
     * @param Countee\Model\Payable\Commitment $commitment
     */
    public function __construct(Nette\Database\Context $database, Countee\Model\Payable\Receivable $receivable, Countee\Model\Payable\Commitment $commitment)
    {
        $this->receivable = $receivable;
        $this->commitment = $commitment;
        
        parent::__construct($database);
    }

    /**
     * Get Account Meta by Key
     *
     * @param string $metaKey
     * @param int $accountId
     * @throws Countee\Model\Bank\Exception\InvalidAccountMetaException
     * @return string
     */
    public function getAccountMeta(string $metaKey, int $accountId)
    {
        $account = $this->getAccount($accountId);
        if (!$account) {
            throw new Exception\InvalidAccountMetaException('Invalid account ID');
        }

        $meta = $this->getTable('account_meta')->where([
            'account_id' => $accountId,
            'meta_key' => $metaKey
        ]);

        if ($meta->count() > 0) {
            return $meta->fetch()->meta_value;
        } else {
            throw new Exception\InvalidAccountMetaException('Invalid Account meta with key: ' . $metaKey . '.');
        }
    }
    
    /**
     * Set Account Meta by Key
     * 
     * @param string $metaKey
     * @param string $metaValue
     * @param int $accountId
     * @throws Exception\InvalidAccountMetaException
     */
    public function setAccountMeta(string $metaKey, string $metaValue, int $accountId)
    {
        
        $account = $this->getAccount($accountId);
                
        if (!$account) {
            throw new Exception\AccountDoesNotExistsException('Invalid account ID');
        }
        
        $meta = $this->getTable('account_meta')->where([
            'account_id' => $accountId,
            'meta_key' => $metaKey
        ]);
                
        if ($meta->count() > 0) {
            $meta->update([
                'meta_value' => $metaValue
            ]);
        } else {
            $this->getTable('account_meta')->insert([
                'meta_key' => $metaKey,
                'meta_value' => $metaValue,
                'account_id' => $accountId
            ]);
        }
    }
    
    public function getAccountName(int $accountId, $userId = null)
    {
        $account = $this->getAccount($accountId, $userId);
        return $account->account_name;
    }
    
    /**
     * Synchronize account transactions with Countee
     *
     * @param int $accountId
     * @return void
     */
    public function syncAccount(int $accountId)
    {
        $syncedTransactions = $this->makeSyncAccount($accountId);
        
        foreach ($syncedTransactions as $transaction) {
            $this->receivable->syncReceivable($accountId, $transaction->getVs(), $transaction->getAmmount());
            $this->commitment->syncCommitment($accountId, $transaction->getAccountNumber(), $transaction->getAccountBank(), $transaction->getAmmount(), $transaction->getVs());
        }

        return $syncedTransactions;
    }
    
    /**
     * Bank synchronize account implementation
     * 
     * @return bool
     */
    abstract protected function makeSyncAccount(int $accountId);
    
    public function payment($sourceAccountId, $destAccountNumber, $destBankCode, $ammount, $vs, $ks, $ss, $msg)
    {
        return $this->makePayment($sourceAccountId, $destAccountNumber, $destBankCode, $ammount, $vs, $ks, $ss, $msg);
    }

    /**
     * Bank payment implementation
     * 
     * @return bool
     */
    abstract protected function makePayment($sourceAccountId, $destAccountNumber, $destBankCode, $ammount, $vs, $ks, $ss, $msg) : bool;
    
    
    public static function getAccountPrefix($accountNumber)
    {
        $accountNumberString = (string) $accountNumber;
        if(strpos($accountNumberString, '-')) {
            return explode('-', $accountNumberString);
        }
        
        if (strlen($accountNumberString) > 10) {
            return substr($accountNumberString, 0, -10);
        }
        
        return null;
    }
    
    public static function getAccountNumber($accountNumber)
    {
        $accountNumberString = (string) $accountNumber;
        if(strpos($accountNumberString, '-')) {
            return explode('-', $accountNumberString);
        }
        
        if (strlen($accountNumberString) > 10) {
            return substr($accountNumberString, 0, -10);
        }
        
        return null;
    }

}
