<?php

namespace Countee\Model\Bank\Fio;

use Countee;
use GuzzleHttp;

class Account extends Countee\Model\Bank\Account
{

    const BANK_ID = 1;

    private function requestLastTransactions(int $accountId)
    {
        $account = $this->getTable('account')->get($accountId);
        $downloadFrom = $account->last_download;

        if ((int) $downloadFrom->format('Y') < 1990) {
            $downloadFrom = new \DateTime('1990-12-21 00:00:00');
        }

        $downloadTo = new \DateTime();

        if (($downloadTo->getTimestamp() - $downloadFrom->getTimestamp()) < self::SYNC_LIMIT) {
            throw new \Countee\Model\Bank\Exception\SyncLimitException;
        }

        // Prevents skipping some transaction
        $downloadFrom->modify('-1 day');

        $client = new \GuzzleHttp\Client();
        try {
            $fioToken = $this->getAccountMeta('fio_token', $accountId);
            $response = $client->request('GET', 'https://www.fio.cz/ib_api/rest/periods/' . $fioToken . '/' . $downloadFrom->format('Y-m-d') . '/' . $downloadTo->format('Y-m-d') . '/transactions.json');
            $body = $response->getBody();
            $stringBody = (string) $body;

            $transactionArray = json_decode($stringBody, true);

            return $this->validateInput($transactionArray);
        } catch (GuzzleHttp\Exception\ClientException $e) {
            if ($e->getCode() == 409) {
                throw new Countee\Model\Bank\Exception\RequestLimitException;
            } else {
                throw new Countee\Model\Bank\Exception\GeneralSyncException('Something went wrong.');
            }
        } catch (GuzzleHttp\Exception\ServerException $e) {
            throw new Countee\Model\Bank\Exception\GeneralSyncException('Something went wrong.');
        }
    }

    private function validateInput($transactionArray)
    {
        if (!is_array($transactionArray)) {
            throw new Countee\Model\Bank\Exception\InvalidStructure;
        }

        if (!isset($transactionArray['accountStatement']) || !is_array($transactionArray['accountStatement'])) {
            throw new Countee\Model\Bank\Exception\InvalidStructure;
        }

        $accountStatement = $transactionArray['accountStatement'];

        // Check statement info
        if (!isset($accountStatement['info']['accountId']) || !isset($accountStatement['info']['bankId'])) {
            throw new Countee\Model\Bank\Exception\InvalidStructure;
        }

        // Check statement transactions
        if (!isset($accountStatement['transactionList']['transaction']) || !is_array($accountStatement['transactionList']['transaction'])) {
            throw new Countee\Model\Bank\Exception\InvalidStructure;
        }

        return $transactionArray;
    }

    /**
     * Synchronize account transactions with Countee
     *
     * @param int $accountId
     * @return void
     */
    protected function makeSyncAccount(int $accountId)
    {

        $lastTransactionsArray = $this->requestLastTransactions($accountId);
        $syncedTransactions = [];
        
        $transactionSync = new Countee\Model\Bank\TransactionSync($this->database, $accountId);

        $syncCounter = 0;
        foreach ($lastTransactionsArray['accountStatement']['transactionList']['transaction'] as $transaction) {
            $transactionObj = new Transaction($transaction);

            // If valid then sync with DB
            if ($transactionObj->inputValid()) {
                if ($transactionSync->transfer($transactionObj, $accountId)) {
                    $syncedTransactions[] = $transactionObj;
                }
            }
        }

        $this->getTable('account')->get($accountId)->update([
            'last_download' => new \DateTime,
            'account_number' => $lastTransactionsArray['accountStatement']['info']['accountId'],
            'bank_code' => $lastTransactionsArray['accountStatement']['info']['bankId']
        ]);

        return $syncedTransactions;
    }

    protected function makePayment($sourceAccountId, $destAccountNumber, $destBankCode, $ammount, $vs, $ks, $ss, $msg): bool
    {

        $account = $this->getAccount($sourceAccountId);
        $fioToken = $this->getAccountMeta('fio_token', $account->id);
        $xmlFile = tmpfile();
        
        $xmlString = '<?xml version="1.0" encoding="UTF-8"?>
        <Import xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xsi:noNamespaceSchemaLocation="http://www.fio.cz/schema/importIB.xsd">
        <Orders>
        <DomesticTransaction>
        <accountFrom>' . $account->account_number . '</accountFrom>
        <currency>CZK</currency>
        <amount>' . $ammount . '</amount>
        <accountTo>' . $destAccountNumber . '</accountTo>
        <bankCode>' . $destBankCode . '</bankCode>
        <ks>' . $ks . '</ks>
        <vs>' . $vs . '</vs>
        <ss>' . $ss . '</ss>
        <date>' . date("Y-m-d") . '</date>
        <messageForRecipient>' . $msg . '</messageForRecipient>
        <comment></comment>
        <paymentType>431001</paymentType>
        </DomesticTransaction>
        </Orders>
        </Import>';

        fwrite($xmlFile, $xmlString);

        $client = new \GuzzleHttp\Client();

        try {
            $response = $client->request('POST', 'https://www.fio.cz/ib_api/rest/import/', [
                'multipart' => [
                    [
                        'name' => 'type',
                        'contents' => 'xml'
                    ],
                    [
                        'name' => 'token',
                        'contents' => $fioToken
                    ],
                    [
                        'name' => 'lng',
                        'contents' => 'cs'
                    ],
                    [
                        'name' => 'file',
                        'contents' => $xmlFile
                    ],
                ]
            ]);
            
        } catch (GuzzleHttp\Exception\ClientException $e) {
            if ($e->getCode() == 409) {
                throw new Countee\Model\Bank\Exception\RequestLimitException;
            } else {
                throw new Countee\Model\Bank\Exception\GeneralSyncException('Something went wrong.');
            }
        } catch (GuzzleHttp\Exception\ServerException $e) {
            throw new Countee\Model\Bank\Exception\GeneralSyncException('Something went wrong.');
        }

        return true;
    }

    /**
     * Creates Fio account
     * 
     * @param int $userId
     * @param string $accountName
     * @param string $accountDescription
     * @param string $fioToken
     * @return type
     * @throws Countee\Model\Exception\UserDoesNotExistsException
     */
    public function createAccount(int $userId, string $accountName, string $accountDescription, string $fioToken)
    {
        $user = $this->getTable('user')->get($userId);
        if (!$user) {
            throw new Countee\Model\Exception\UserDoesNotExistsException;
        }

        $account = $this->getTable('account')->insert([
            'bank_id' => static::BANK_ID,
            'user_id' => $userId,
            'account_name' => $accountName,
            'account_description' => $accountDescription,
        ]);

        $this->setAccountMeta('fio_token', $fioToken, $account->id);
        
        try {
            $this->syncAccount($account->id);
            return $account;
        } catch (Countee\Model\Bank\Exception\GeneralSyncException $e) {
            $account->delete();
            return false;
        }


    }

    /**
     * Fio account edit
     * 
     * @param int $accountId
     * @param int $userId
     * @param string $accountName
     * @param string $accountDescription
     * @param string $fioToken
     * @throws Countee\Model\Exception\UserDoesNotExistsException
     */
    public function editAccount(int $accountId, int $userId, string $accountName, string $accountDescription, string $fioToken)
    {
        $user = $this->getTable('user')->get($userId);
        if (!$user) {
            throw new Countee\Model\Exception\UserDoesNotExistsException;
        }

        $account = $this->getAccount($accountId, $userId);

        $account->update([
            'account_name' => $accountName,
            'account_description' => $accountDescription,
        ]);

        $this->setAccountMeta('fio_token', $fioToken, $account->id);
    }

}
