<?php

namespace Countee\Model\Payable;

use Higi90\SpaydParser;
use Countee;

class Commitment extends Base
{
    
    const STATUS_CREATED = 1;
    const STATUS_SUBMITTED = 2;
    const STATUS_PAID = 3;

    /**
     * Creates commitent using spayd string
     * 
     * @param int $userId
     * @param $invoice
     * @param type $name
     * @param type $description
     * @param type $accountId
     */
    public function createCommitmentInvoice(int $userId, $invoice, $name = null, $description = null, $accountId = null)
    {
       $user = $this->getUser($userId);
       $pdfFile = TEMP_DIR . '/invoice/'.uniqid('invoice', true);
       $invoice->move($pdfFile);
       
       $parse = new Countee\Model\Invoice\Parse($pdfFile);

       $this->createCommitmentSpayd($userId, $parse->getSpayd(), $name, $description, $accountId);
    }
    
    /**
     * Creates commitent using spayd string
     * 
     * @param int $userId
     * @param string $spaydString
     * @param type $name
     * @param type $description
     * @param type $accountId
     */
    public function createCommitmentSpayd(int $userId, string $spaydString, $name = null, $description = null, $accountId = null)
    {
        $user = $this->getUser($userId);

        $spayd = new SpaydParser\Spayd($spaydString);
        $spaydAccount = $spayd->getAccount();
        
        if (!$spaydAccount instanceof SpaydParser\AccountCzech) {
            throw new Countee\Model\Exception\InternationalAccountException;
        }


        $values = \Nette\Utils\ArrayHash::from([
                    'name' => $name,
                    'description' => $description,
                    'account_id' => $accountId,
                    'ammount' => $spayd->getParam('AM'),
                    'message' => $spayd->getParam('MSG'),
                    'vs' => $spayd->getParam('X-VS'),
                    'ks' => $spayd->getParam('X-KS'),
                    'ss' => $spayd->getParam('X-SS'),
                    'account_number' => $spaydAccount->getAccountNumber(),
                    'account_bank' => $spaydAccount->getBankCode(),
                    'due_date' => $spayd->getDueDate(),
                    'status' => self::STATUS_CREATED
        ]);

        $this->createCommitment($userId, $values);
    }

    /**
     * Creates commitment record
     * 
     * @param int $userId
     * @param type $values
     * @return type
     */
    public function createCommitment(int $userId, $values)
    {

        $user = $this->getUser($userId);

        if (!empty($values->account_id)) {
            $this->getAccount($values->account_id, $userId);
        }

        if($values->due_date) {
            if(!is_object($values->due_date) || !($values->due_date instanceof \Datetime)) {
                $dueDate = \DateTime::createFromFormat('d. m. Y', $values->due_date);
            } else {
                $dueDate = $values->due_date;
            }  
        } else {
            $dueDate = new \DateTime;
        }

        unset($values->due_date);

        $dbValues = array_merge([
            'user_id' => $user->id,
            'due_date' => $dueDate,
            'status' => self::STATUS_CREATED
                ], (array) $values);
        $commitment = $this->getTable('commitment')->insert($dbValues);

        return $commitment;
    }

    /**
     * Edits commitment record
     * 
     * @param int $userId
     * @param int $commitmentId
     * @param type $values
     * @return type
     */
    public function editCommitment(int $userId, int $commitmentId, $values)
    {
        $user = $this->getUser($userId);
        $commitment = $this->getCommitment($commitmentId, $userId);

        if (!empty($values->account_id)) {
            $this->getAccount($values->account_id, $userId);
        }

        if ($commitment->paid >= $commitment->ammount) {
            $commitment->update(['name' => $values->name, 'description' => $values->description]);
            return $commitment;
        }

        $dueDate = $date = \DateTime::createFromFormat('d. m. Y', $values->due_date);

        unset($values->due_date);

        $dbValues = array_merge([
            'user_id' => $user->id,
            'due_date' => $dueDate,
                ], (array) $values);

        $commitment->update($dbValues);

        return $commitment;
    }

    /**
     * Gets commitments selection
     * 
     * @param int $userId
     * @param type $accountId
     * @return type
     */
    public function getCommitments(int $userId, $accountId = null)
    {
        $user = $this->getUser($userId);
        $commitments = $user->related('commitment');

        if (!empty($accountId)) {
            $commitments = $commitments->where(['account_id' => $accountId]);
        }

        return $commitments;
    }
    
    public function pay(int $userId, int $commitmentId) {
        $commitment = $this->getCommitment($commitmentId, $userId);
        
        if ($commitment->account_id < 1) {
            throw new Exception\AccountNotSetException;
        }
        
        if($commitment->status != self::STATUS_CREATED) {
            throw new \Nette\Application\BadRequestException;
        }
        
        $bankTag = $commitment->account->bank->tag;
        $accountModel = $this->container->getByType('\Countee\Model\Bank\\' . ucfirst(strtolower($bankTag)) . '\Account');
    
        $payment = $accountModel->payment($commitment->account_id, $commitment->account_number, $commitment->account_bank, $commitment->ammount, $commitment->vs, $commitment->ks, $commitment->ss, $commitment->message);
    
        if ($payment) {
            $commitment->update([
                'status' => self::STATUS_SUBMITTED
            ]);
            
            return true;
        } else {
            return false;
        }
    }
    
    public function deleteCommitment(int $commitmentId, int $userId)
    {
        $commitment = $this->getCommitment($commitmentId, $userId);
        return $commitment->delete();
    }
    
    /**
     * 
     * @param int $accountId
     * @param type $vs
     */
    public function syncCommitment(int $accountId, $refAccountNumber, $refBankCode, $ammount, $vs = null)
    {
        $ammount = $ammount * -1;
        
        $commitments = $this->getTable('commitment')->where([
            'account_id' => $accountId,
            'account_number' => $refAccountNumber,
            'account_bank' => $refBankCode
        ]);
        
        if (!empty($vs)) {
            $commitments = $commitments->where(['vs' => $vs]);
        }
        
        foreach ($commitments as $commitment) {
            if($commitment->ammount > $commitment->paid) {
                $paid = (float)$commitment->paid + (float)$ammount;
                
                if($paid >= (float)$commitment->ammount) {
                    $commitment->update([
                        'paid' => $paid,
                        'status' => self::STATUS_PAID
                    ]);
                } else {
                    $commitment->update([
                        'paid' => $paid,
                    ]);
                }
            }
        }
    }

}
