<?php

namespace Countee\Model;

use Nette;
use Nette\Utils\Image;

/**
 * User management.
 */
class UserManager extends BaseDatabaseModel
{
    const TABLE_NAME = 'user';
    const COLUMN_ID = 'id';
    const COLUMN_NAME = 'name';
    const COLUMN_EMAIL = 'email';
    const COLUMN_CREATED = 'created';
    const COLUMN_LAST_LOGIN = 'last_login';
    const COLUMN_FACEBOOK_ID = 'fb_id';
    const COLUMN_FACEBOOK_ACCESS_TOKEN = 'fb_access_token';
    const FACEBOOK_REQUIRED_COLUMNS = ['id', 'name', 'email', 'picture'];

    /**
     * @var \Countee\Model\Category\Seed
     */
    private $categorySeed; 
    
    public function __construct(Nette\Database\Context $database, Category\Seed $categorySeed) {
        $this->categorySeed = $categorySeed;
        
        parent::__construct($database);
    }
    
    /**
     * Finds user by facebook id
     * @param type $fbId
     * @return boolean
     */
    public function findByFacebookId($fbId)
    {
        $user = $this->getTable(self::TABLE_NAME)->where(self::COLUMN_FACEBOOK_ID, $fbId);
        if ($user->count() == 0) {
            return false;
        } else {
            return $user->fetch();
        }
    }

    /**
     * Register new user using Facebook info
     * @param Nette\Utils\ArrayHash $userInfo
     * @throws Exception\MissingArgsException
     * @return Nette\Database\Table\Selection
     */
    public function registerFromFacebook(Nette\Utils\ArrayHash $userInfo) : Nette\Database\Table\ActiveRow
    {
        foreach (self::FACEBOOK_REQUIRED_COLUMNS as $requiredColumn) {
            if (!array_key_exists($requiredColumn, $userInfo)) {
                throw new Exception\MissingArgsException;
            }
        }

        $this->updateFacebookImage($userInfo['id'], $userInfo['picture']['data']['url']);

        $user = $this->getTable('user')->insert([
                    self::COLUMN_NAME => $userInfo['name'],
                    self::COLUMN_EMAIL => $userInfo['email'],
                    self::COLUMN_CREATED => new \DateTime,
                    self::COLUMN_FACEBOOK_ID => $userInfo['id']
        ]);
        
        // Seed user transaction / payable categories
        $this->categorySeed->seedCategories($user);
        
        return $user;
    }

    /**
     * Updates user Facebook access token
     * @param int $fbId
     * @param string $fbAccessToken
     * @return boolean
     */
    public function updateFacebookAccessToken(int $fbId, string $fbAccessToken) : bool
    {
        $user = $this->findByFacebookId($fbId);

        if (!$user) {
            return false;
        }

        return $user->update([
                    self::COLUMN_FACEBOOK_ACCESS_TOKEN => $fbAccessToken,
                    self::COLUMN_LAST_LOGIN => new \DateTime
        ]);
    }

    /**
     * Updates profile avatar from Facebook
     *
     * @param int $fbId
     * @param string $pictureUrl
     * @return void
     */
    public function updateFacebookImage(int $fbId, string $pictureUrl)
    {
        $ch = curl_init($pictureUrl);
        $fp = fopen(__DIR__ . '/../../www/profile/' . $fbId . '.jpg', 'wb');
        curl_setopt($ch, CURLOPT_FILE, $fp);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_exec($ch);
        curl_close($ch);
        fclose($fp);
    }
}
